@@ -1,4 +1,4 @@
-use std::{collections::BTreeMap, ops::Add};
+use std::{collections::BTreeMap, ops::Add, option::Option::None};
 
 use mit_commit::{CommitMessage, Trailer};
 
@@ -49,11 +49,36 @@ pub(crate) fn lint(commit: &CommitMessage) -> Option<Problem> {
     if duplicated_trailers.is_empty() {
         None
     } else {
+        let commit_text = String::from(commit.clone());
         let warning = warning(&duplicated_trailers);
         Some(Problem::new(
             ERROR.into(),
             warning,
             Code::DuplicatedTrailers,
+            commit,
+            Some(
+                duplicated_trailers
+                    .into_iter()
+                    .flat_map(|trailer| {
+                        commit_text
+                            .match_indices(&trailer)
+                            .skip(1)
+                            .map(|x| {
+                                (
+                                    format!("Duplicated `{}`", trailer),
+                                    x.0,
+                                    commit_text
+                                        .chars()
+                                        .skip(x.0)
+                                        .take_while(|x| x != &'\n')
+                                        .count(),
+                                )
+                            })
+                            .collect::<Vec<_>>()
+                    })
+                    .collect(),
+            ),
+            None,
         ))
     }
 }
@@ -77,7 +102,10 @@ fn warning(duplicated_trailers: &[String]) -> String {
 mod tests_has_duplicated_trailers {
     #![allow(clippy::wildcard_imports)]
 
+    use std::option::Option::None;
+
     use indoc::indoc;
+    use miette::{GraphicalReportHandler, GraphicalTheme, Report};
     use mit_commit::CommitMessage;
 
     use super::*;
@@ -100,20 +128,20 @@ mod tests_has_duplicated_trailers {
 
     #[test]
     fn two_duplicates() {
+        let message = indoc!(
+            "
+            An example commit
+
+            This is an example commit without any duplicate trailers
+
+            Signed-off-by: Billie Thompson <email@example.com>
+            Signed-off-by: Billie Thompson <email@example.com>
+            Co-authored-by: Billie Thompson <email@example.com>
+            Co-authored-by: Billie Thompson <email@example.com>
+            "
+        );
         test_lint_duplicated_trailers(
-            indoc!(
-                "
-                An example commit
-
-                This is an example commit without any duplicate trailers
-
-                Signed-off-by: Billie Thompson <email@example.com>
-                Signed-off-by: Billie Thompson <email@example.com>
-                Co-authored-by: Billie Thompson <email@example.com>
-                Co-authored-by: Billie Thompson <email@example.com>
-                "
-            )
-            .into(),
+            message.into(),
             &Some(Problem::new(
                 ERROR.into(),
                 "These are normally added accidentally when you\'re rebasing or amending to a \
@@ -121,15 +149,28 @@ mod tests_has_duplicated_trailers {
                  this by deleting the duplicated \"Co-authored-by\", \"Signed-off-by\" fields"
                     .into(),
                 Code::DuplicatedTrailers,
+                &message.into(),
+                Some(vec![
+                    (
+                        "Duplicated `Co-authored-by`".to_string(),
+                        231_usize,
+                        51_usize,
+                    ),
+                    (
+                        "Duplicated `Signed-off-by`".to_string(),
+                        128_usize,
+                        50_usize,
+                    ),
+                ]),
+                None,
             )),
         );
     }
 
     #[test]
     fn signed_off_by() {
-        test_lint_duplicated_trailers(
-            indoc!(
-                "
+        let message = indoc!(
+            "
                 An example commit
 
                 This is an example commit without any duplicate trailers
@@ -137,8 +178,9 @@ mod tests_has_duplicated_trailers {
                 Signed-off-by: Billie Thompson <email@example.com>
                 Signed-off-by: Billie Thompson <email@example.com>
                 "
-            )
-            .into(),
+        );
+        test_lint_duplicated_trailers(
+            message.into(),
             &Some(Problem::new(
                 ERROR.into(),
                 "These are normally added accidentally when you\'re rebasing or amending to a \
@@ -146,15 +188,17 @@ mod tests_has_duplicated_trailers {
                  this by deleting the duplicated \"Signed-off-by\" field"
                     .into(),
                 Code::DuplicatedTrailers,
+                &message.into(),
+                Some(vec![("Duplicated `Signed-off-by`".to_string(), 128, 50)]),
+                None,
             )),
         );
     }
 
     #[test]
     fn co_authored_by() {
-        test_lint_duplicated_trailers(
-            indoc!(
-                "
+        let message = indoc!(
+            "
                 An example commit
 
                 This is an example commit without any duplicate trailers
@@ -162,8 +206,9 @@ mod tests_has_duplicated_trailers {
                 Co-authored-by: Billie Thompson <email@example.com>
                 Co-authored-by: Billie Thompson <email@example.com>
                 "
-            )
-            .into(),
+        );
+        test_lint_duplicated_trailers(
+            message.into(),
             &Some(Problem::new(
                 ERROR.into(),
                 "These are normally added accidentally when you\'re rebasing or amending to a \
@@ -171,15 +216,17 @@ mod tests_has_duplicated_trailers {
                  this by deleting the duplicated \"Co-authored-by\" field"
                     .into(),
                 Code::DuplicatedTrailers,
+                &message.into(),
+                Some(vec![("Duplicated `Co-authored-by`".to_string(), 129, 51)]),
+                None,
             )),
         );
     }
 
     #[test]
     fn relates_to() {
-        test_lint_duplicated_trailers(
-            indoc!(
-                "
+        let message = indoc!(
+            "
                 An example commit
 
                 This is an example commit without any duplicate trailers
@@ -187,8 +234,9 @@ mod tests_has_duplicated_trailers {
                 Relates-to: #315
                 Relates-to: #315
                 "
-            )
-            .into(),
+        );
+        test_lint_duplicated_trailers(
+            message.into(),
             &Some(Problem::new(
                 ERROR.into(),
                 "These are normally added accidentally when you\'re rebasing or amending to a \
@@ -196,6 +244,9 @@ mod tests_has_duplicated_trailers {
                  this by deleting the duplicated \"Relates-to\" field"
                     .into(),
                 Code::DuplicatedTrailers,
+                &message.into(),
+                Some(vec![("Duplicated `Relates-to`".to_string(), 94, 16)]),
+                None,
             )),
         );
     }
@@ -267,4 +318,60 @@ mod tests_has_duplicated_trailers {
             expected, actual
         );
     }
+
+    #[test]
+    fn formatting() {
+        let message = indoc!(
+            "
+            An example commit
+
+            This is an example commit without any duplicate trailers
+
+            Signed-off-by: Billie Thompson <email@example.com>
+            Signed-off-by: Billie Thompson <email@example.com>
+            Co-authored-by: Billie Thompson <email@example.com>
+            Co-authored-by: Billie Thompson <email@example.com>
+            "
+        );
+        let problem = lint(&CommitMessage::from(message.to_string()));
+        let actual = fmt_report(&Report::new(problem.unwrap()));
+        let expected = indoc!(
+            "
+            DuplicatedTrailers
+            
+              \u{d7} Your commit message has duplicated trailers
+               \u{256d}\u{2500}[5:1]
+             5 \u{2502} Signed-off-by: Billie Thompson <email@example.com>
+             6 \u{2502} Signed-off-by: Billie Thompson <email@example.com>
+               \u{b7} \u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{252c}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}
+               \u{b7}                          \u{2570}\u{2500}\u{2500} Duplicated `Signed-off-by`
+             7 \u{2502} Co-authored-by: Billie Thompson <email@example.com>
+             8 \u{2502} Co-authored-by: Billie Thompson <email@example.com>
+               \u{b7} \u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{252c}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}\u{2500}
+               \u{b7}                          \u{2570}\u{2500}\u{2500} Duplicated `Co-authored-by`
+               \u{2570}\u{2500}\u{2500}\u{2500}\u{2500}
+              help: These are normally added accidentally when you're rebasing or
+                    amending to a commit, sometimes in the text editor, but often by
+                    git hooks.
+                    
+                    You can fix this by deleting the duplicated \"Co-authored-by\",
+                    \"Signed-off-by\" fields
+            "
+        )
+        .to_string();
+        assert_eq!(
+            actual, expected,
+            "Message {:?} should have returned {:?}, found {:?}",
+            message, expected, actual
+        );
+    }
+
+    fn fmt_report(diag: &Report) -> String {
+        let mut out = String::new();
+        GraphicalReportHandler::new_themed(GraphicalTheme::unicode_nocolor())
+            .with_width(80)
+            .render_report(&mut out, diag.as_ref())
+            .unwrap();
+        out
+    }
 }
